%------------------------------------------------------------------------------
\section{Options}
%------------------------------------------------------------------------------

\paragraph{Usage}

\begin{verbatim}
ghci -Wall -package rh -package util Options.lhs
\end{verbatim}

%if False
%- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
\subsection{Module header}
%- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

> module Options (module Options, module System.Console.GetOpt)
> where
> import System.Console.GetOpt
> import Data.List
> -- import Char
> --import IO

%endif

%- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
\subsection{Options}
%- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

 data Flag  =  Bare
            |  Copying
            |  Dump [Int]
            |  Help
            |  Interactive
            |  LetFile String --FileName
            |  Pagewidth Int
            |  Trace
            |  Verbose
            |  Version
            |  Warranty
               deriving (Eq)

> data Options    =  Flags
>                      {  bare         ::  Bool,
>                         dumpphases   ::  [Int],
>                         interactive  ::  Bool,
>                         letfile      ::  [String],
>                         pagewidth    ::  Int,
>                         trace        ::  Bool,
>                         typechecker  ::  Bool,
>                         verbose      ::  Bool,
>                         help         ::  Bool,
>                         copying      ::  Bool,
>                         history      ::  String,
>                         version      ::  Bool,
>                         warranty     ::  Bool  }

> defaultOptions  =  Flags
>                      {  bare         =   False,
>                         dumpphases   =   [],
>                         interactive  =   False,
>                         letfile      =   [],
>                         pagewidth    =   80,
>                         trace        =   False,
>                         typechecker  =   True,
>                         verbose      =   False,
>                         help         =   False,
>                         copying      =   False,
>                         history      =   "",
>                         version      =   False,
>                         warranty     =   False }

-fomit-frame-pointer

> optionDescrs  ::  [OptDescr (Options -> Options)]
> optionDescrs  
>   =   [ Option "b"   ["bare"]       (NoArg (\o -> o { bare = True }))         "don't preload `System'",
>         Option ""    ["copying"]    (NoArg (\o -> o { copying = True }))      "display details of copying",
>         Option "d"   ["dump"]       (OptArg getPass "nat")                    "dump given pass (default: dump all)",
>         Option "?"   ["help"]       (NoArg (\o -> o { help = True }))         "print this information",
>         Option "h"   ["history"]    (OptArg getHistory "file")                "load history file",
>         Option "i"   ["interactive"](NoArg (\o -> o { interactive = True }))  "enter command line interpreter",
>         Option "l"   ["let"]        (ReqArg (\f o -> o { letfile = letfile o ++ [f] }) "file")
>                                                                               "load library",
>         Option "p"   ["pagewidth"]  (OptArg getPagewidth "nat")  "use the specified pagewidth for pretty printing (default: 80)",
>         Option "t"   ["trace"]      (NoArg (\o -> o { trace = True }))        "enter interactive tracer",
>         Option "n"   ["notypes"]    (NoArg (\o -> o { typechecker = False })) "run without typechecker",
>         Option "v"   ["verbose"]    (NoArg (\o -> o { verbose = True}))       "be verbose",
>         Option ""    ["version"]    (NoArg (\o -> o { version = True}))       "print version information",
>         Option ""    ["warranty"]   (NoArg (\o -> o { warranty = True}))      "display details of warranty" ]

> getOptions :: [String] -> (Options, [String], [String])
> getOptions argv =  (\ (os, x, y)  ->  (foldl (flip ($)) defaultOptions os, x, y)) $
>                    getOpt Permute optionDescrs argv

> getPass, getPagewidth :: Maybe String -> (Options -> Options)
>
> getPass Nothing   =  \o -> o { dumpphases = [1 .. 9] }
> getPass (Just s)  =  \o -> o { dumpphases = read s : dumpphases o }
>
> getPagewidth Nothing   =  \o -> o { pagewidth = 80 }
> getPagewidth (Just s)  =  \o -> o { pagewidth = read s }
>
> getHistory Nothing   =  \o -> o { history = ".history" }
> getHistory (Just s)  =  \o -> o { history = s }

 toggle :: Flag -> [Flag] -> [Flag]
 toggle x xs = if x `elem` xs then delete x xs else x:xs

 (<<)                          :: String -> String -> Bool
 s << t                        =  map toLower s `isPrefix` map toLower t

 isPrefix                      :: (Eq a) => [a] -> [a] -> Bool
 p `isPrefix` as               =  p == take (length p) as