\documentclass[fleqn]{report}

\usepackage[english]{babel}
\usepackage[latin1]{inputenc}
\usepackage{xspace}
\usepackage{url}
\usepackage{color}

%include lhs2TeX.fmt

\title{BPL: Rationale}
\author{Ralf Hinze}

%==============================================================================

\setlength{\mathindent}{\parindent}

\newcommand{\technical}[1]{\textsl{#1}}
\newcommand{\ralf}[1]{[\textit{Ralf says:}~#1]}
\newcommand{\andres}[1]{[\textit{Andres says:}~#1]}
%\newcommand{\ralf}[1]{}
%\newcommand{\andres}[1]{}

%==============================================================================

%subst keyword a = a
%format array     = "{\bfseries array}"
%format case      = "{\bfseries case}"
%format catch     = "{\bfseries catch}"
%format class     = "{\bfseries class}"
% format cont      = "{\bfseries cont}"
%format continue  = "{\bfseries continue}"
%format data      = "{\bfseries data}"
%format do        = "{\bfseries do}"
%format else      = "{\bfseries else}"
%format end       = "{\bfseries end}"
%format exception = "{\bfseries exception}"
%format forall    = "{\bfseries forall}"
%format fun       = "{\bfseries fun}"
%format function  = "{\bfseries function}"
%format functor   = "{\bfseries functor}"
%format if        = "{\bfseries if}"
%format in        = "{\bfseries in}"
%format interface = "{\bfseries interface}"
%format inherit   = "{\bfseries inherit}"
%format let       = "{\bfseries let}"
%format letcc     = "{\bfseries letcc}"
%format local     = "{\bfseries local}"
%format method    = "{\bfseries method}"
%format module    = "{\bfseries module}"
%format new       = "{\bfseries new}"
%format object    = "{\bfseries object}"
%format of        = "{\bfseries of}"
%format open      = "{\bfseries open}"
%format rec       = "{\bfseries rec}"
%format ref       = "{\bfseries ref}"
%format sig       = "{\bfseries sig}"
%format struct    = "{\bfseries struct}"
%format then      = "{\bfseries then}"
%format throw     = "{\bfseries throw}"
%format try       = "{\bfseries try}"
%format type      = "{\bfseries type}"
%format val       = "{\bfseries val}"
%format while     = "{\bfseries while}"
%format ...       = "\ldots "

%==============================================================================

\begin{document}

\maketitle

\begin{abstract}
Bla bla \ldots
\end{abstract}

\tableofcontents

%==============================================================================
\chapter{Open issues}%
\label{cha:open-issues}
%==============================================================================

Untertypen: |Nat <: Int|, |Char <: String|.

Bessere Schlsselwrter an Stelle von |struct| und |sig|?

Zustzliche Primitive: |random|, Komposition von Funktionen.

%==============================================================================
\chapter{Lexical syntax}%
\label{cha:lexical-syntax}
%==============================================================================

%==============================================================================
\chapter{Syntax}%
\label{cha:syntax}
%==============================================================================

Klassen: Modulsyntax ist der Recordsyntax vorzuziehen: |method set (i
: Nat) =| statt |set = fun (i : Nat) =>|.

Klassen: |class| hat die gleiche Syntax wie |function|
(Funktionsabstraktion); |new| und |inherit| haben die gleiche Syntax
wie die Funktionsapplikation. OLD.

Operatoren: `| |||| |' statt `| || |' fr die Disjunktion, da `| || |'
schon bei |case|-Ausdrcken verwendet wird:
%
\begin{code}
case e of
         Nil       => e1
       | Cons cons => e2
end
\end{code}
%
Wird `| || |' fr die Disjunktion verwendet, wird die erste
Alternative als |e1 || Cons cons| geparst.

Arrays: Subtil: |e [i]| ist die Subskription; |e ([i])| bezeichnet die
Applikation von |e| auf das 1-elementige Feld |[i]|.

%==============================================================================
\chapter{Values}%
\label{cha:values}
%==============================================================================

%------------------------------------------------------------------------------
\section{Booleans}
\label{sec:booleans}
%------------------------------------------------------------------------------

%------------------------------------------------------------------------------
\section{Natural numbers}
\label{sec:natural-numbers}
%------------------------------------------------------------------------------

%------------------------------------------------------------------------------
\section{Strings}
\label{sec:strings}
%------------------------------------------------------------------------------

%------------------------------------------------------------------------------
\section{Bindings and local definitions}
\label{sec:bindings}
%------------------------------------------------------------------------------

%------------------------------------------------------------------------------
\section{Functions}
\label{sec:functions}
%------------------------------------------------------------------------------

%------------------------------------------------------------------------------
\section{Recursion}
\label{sec:recursion}
%------------------------------------------------------------------------------

%------------------------------------------------------------------------------
\section{Records}
\label{sec:records}
%------------------------------------------------------------------------------

%------------------------------------------------------------------------------
\section{Variants and patterns}
\label{sec:variants}
%------------------------------------------------------------------------------

Layered patterns `|@|' (not in the language) are submsumed by
conjunctive patterns `|&|' (in the language).
%
\begin{code}
x@Cons { head = x, tail = xs }
x&Cons { head = x, tail = xs }
\end{code}

%------------------------------------------------------------------------------
\section{Polymorphism}
\label{sec:polymorphism}
%------------------------------------------------------------------------------

%------------------------------------------------------------------------------
\section{Arrays}
\label{sec:arrays}
%------------------------------------------------------------------------------

Keine mehr-dimensionalen Felder: zu aufwndig.

$n$-dimensionale Felder: endliche Abbildungen $\mathbb{N}^n
\to_{\mathrm{fin}} Val$. Typ: |Array n t| oder |t [_, .., _]| bzw.
mit Angabe der Dimension |t [d1, .., dn]|. Introduktion: |array [e1,
.., en] x1, .., xn => e|. Elimination: |e [e1, .., en]| und |dim e|
(gibt ein Tupel zurck). Unklar: Introduktion durch Aufzhlung von
Elementen (|[[0,1],[2,3]]| ist ein geschachteltes Feld).
%
\begin{code}
function sum (a : Nat[_]) : Nat =
  let
    val s = ref 0
  in
    for i = 0 to dim a do
      s := !s + a[i]
    end;
    !s
  end

function mul (a : Nat[_,_], b : Nat[_,_]) : Nat[_,_] =
  let
    val (m,  p) = dim a
    val (p', n) = dim b
  in
    if p == p' then
      array [m, n] i, j =>
        sum (array [p] k => a [i, k] * b [k, j])
    else
      throw Mul
  end
\end{code}
%
Mit expliziten Dimensionen: |dim : t[d1,..,dn] -> (Nat,..,Nat)|.
%
\begin{code}
function sum (a : Nat[n]) : Nat =
  let
    val s = ref 0
  in
    for i = 0 to n do
      s := !s + a[i]
    end;
    !s
  end

function mul (a : Nat[m,p], b : Nat[p,n]) : Nat[m,n] =
  array [m, n] i, j =>
    sum (array [p] k => a [i, k] * b [k, j])
\end{code}
%
Variablen in Typpositionen werden gebunden? Mehrfach auftretende
Variablen auf Gleichheit getestet?

%------------------------------------------------------------------------------
\section{Contracts}
\label{sec:contracts}
%------------------------------------------------------------------------------

%==============================================================================
\chapter{Effects}%
\label{cha:effects}
%==============================================================================

%------------------------------------------------------------------------------
\section{State}
\label{sec:state}
%------------------------------------------------------------------------------

%------------------------------------------------------------------------------
\section{Loops}
\label{sec:loops}
%------------------------------------------------------------------------------

%------------------------------------------------------------------------------
\section{Exceptions}
\label{sec:exceptions}
%------------------------------------------------------------------------------

%------------------------------------------------------------------------------
\section{Continuations}
\label{sec:continuations}
%------------------------------------------------------------------------------

%==============================================================================
\chapter{Modules}%
\label{cha:modules}
%==============================================================================

TODO: parametrisierte Module.
%
\begin{code}
module FiniteSet <a> (Elem : struct compare : (a, a) -> Ordering end) =
  struct
    data Tree = Empty | Node Tree a Tree

    function insert (elem : a, root : Tree) : Tree =
       ...
  end
\end{code}
%
Sharing constraints unntig!?

%------------------------------------------------------------------------------
\section{}
\label{sec:}
%------------------------------------------------------------------------------

%------------------------------------------------------------------------------
\section{}
\label{sec:}
%------------------------------------------------------------------------------

%------------------------------------------------------------------------------
\section{}
\label{sec:}
%------------------------------------------------------------------------------

%==============================================================================
\chapter{Classes}%
\label{cha:classes}
%==============================================================================

TODO: warum sind Klassenausdrcke (|object ... end|) keine normalen
Ausdrcke?

Class system based on a structural type system.

encapsulation

object

class

instance variable/field/field access

method/method invocation/late binding

new/object creation

inheritance/sub-class/super:
gegenwrtige Implementierung $\cong$ flattening.

self/open recursion

access/access control (public, protected, private)

scope/visibility

class variable/class method:
%
\begin{code}
class C =
  let
    val c : ref 0                       // class variable
  in
    object
      val i : ref 0                     // instance variable
    end
  end

\end{code}
%
class method $\cong$ ordinary function

Ohne internal state entspricht eine Klasse einem Modul, |new|
elaboriert die Klasse stets zu dem gleichen Objekt.

|inherit| entspricht |open| bezglich der Sichtbarkeit. |inherit x =|
bindet das erzeugte Objekt an die Wertevariable |x| und entspricht im
wesentlichen |new|.

Multiple inheritance wird untersttzt (mehrere |inherit|s).

Methoden sind im Unterschied zu Funktionen nicht rekursiv, da ein
Methodenaufruf immer die Form |e.m| hat. Methoden binden implizit
|self|.

Redefinition und overriding wird durch den normalen
Bindungsmechanismus realisiert.

Inner classes?

TODO: Methoden als Brger erster Klasse. Typ von Methoden. Method
invocation als Elimination; Method abstraction als Introduktion??

%------------------------------------------------------------------------------
\section{}
\label{sec:}
%------------------------------------------------------------------------------

%------------------------------------------------------------------------------
\section{}
\label{sec:}
%------------------------------------------------------------------------------

%------------------------------------------------------------------------------
\section{Access control}
\label{sec:access-control}
%------------------------------------------------------------------------------

No access control via public, protected, private annotations; access
control only via scope.

I. Instance variable `|x|' is globally visible:
%
\begin{code}
class Counter (i : Nat) =
  object
    val x = ref i // `x' is public
    method inc () : () = x := !x + 1
  end
\end{code}

II. Instance variable `|x|' is \emph{not} visible on the outside
(neither to subclasses nor to clients):
%
\begin{code}
class Counter (i : Nat) =
  object
    local
      val x = ref i // `x' is private
    in
      method get () : Nat = !x
      method inc () : () = x := !x + 1
    end
  end
\end{code}
%
\NB\ ohne |set| ist die Verwendung des Zhlers stark eingeschrnkt.
Unterklassen knnen nicht auf `|x|' zugreifen.

III. Unterscheidung Friends und Clients. Klasse (sprich:
Implementierung) und damit |new| verbergen, aber den Klassentyp
(sprich: Interface/Schnittstelle) und verschiedene Konstruktoren
bekanntgeben.
%
\begin{code}
class Counter (i : Nat) =
  object
    val x = ref i
    method inc () : () = x := !x + 1
  end

type Counter =
  object
    method inc : ()  -> ()
  end

function counter (i : Nat) : Counter =
  new Counter i
\end{code}
%
Die Klasse |Counter| wird mit Hilfe des Modulsystems nicht nach auen
bekanntgegeben. ODER: 2 Klassen definieren, eine einschrnken und nur
die eingeschrnkte nach auen bekanntgeben.
%
\begin{code}
class Private-Counter (i : Nat) =
  object
    val x = ref i
    method inc () : () = x := !x + 1
  end

type Counter =
  object
    method inc : ()  -> ()
  end

class Counter : Counter = Private-Counter
\end{code}
%

%------------------------------------------------------------------------------
\section{Modules versus classes}
\label{sec:modules-versus-classes}
%------------------------------------------------------------------------------

Unterschiede:
%
\begin{itemize}
\item
Mit |new| gelangen Klassen in die Welt der Ausdrcke (werden zu
Objekten) und knnen dann wie normale Werte gehandhabt werden.
\item
Method invocation and dynamic/late binding of @self@.
\item
Unterschied im Zeitpunkt der Abarbeitung:
%
\begin{code}
class C =
  object
    val x = ref 0
  end
\end{code}
%
Der Ausdruck |object ... end| wird verzgert; |new| forciert die
Auswertung des verzgerten Objekts, d.h. `|x|' wird zum Zeitpunkt der
Objekterschaffung mit |new| allokiert (`|x|' entspricht somit einer
Instanzvariablen)..

Im Vergleich dazu:
%
\begin{code}
module M =
  struct
    val x = ref 0
  end
\end{code}
%
Der Ausdruck |struct ... end| wird sofort abgearbeitet (`|x|'
entspricht somit einer Klassen- bzw. Modulvariablen).

\end{itemize}
%
See also, \emph{Using modules as classes} by J\'er\^ome Vouillon.

\end{document}

%------------------------------------------------------------------------------
\section{}
\label{sec:}
%------------------------------------------------------------------------------

